from osgeo import gdal, gdal_array
import sys
import os
import numpy as np
from datetime import datetime


###################################################################################################################
#
# GLOFAS seasonal forecast processing script 3: 
#
# This script takes the weekly exceedance probabilities from script 2
# and computes the maximum probability of exceeding these thresholds during the 4-month forecast horizon
#
#(file paths have been changed)
####################################################################################################################



#Function to open a raster file and put the data into an array
def open_raster_file(fdir, year, month, week, pctl):
    filename = fdir+"prob_"+str(year)+str(month).zfill(2)+"01_w"+str(week+1).zfill(2)+"_p"+str(pctl)+"m.map"
    ff = gdal.Open(filename)
    rows = ff.RasterYSize
    cols = ff.RasterXSize
    band = ff.GetRasterBand(1)
    data = band.ReadAsArray(0,0,cols,rows)
    ff = None # closes the file
    
    return data

#Function to open a raster file and get out the geotransform and projection needed to save an array as a pcraster map
def get_geo_data(fdir, year, month, week, pctl):
    filename = fdir+"prob_"+str(year)+str(month).zfill(2)+"01_w"+str(week+1).zfill(2)+"_p"+str(pctl)+"m.map"
    ff = gdal.Open(filename)
    geotrans = ff.GetGeoTransform()
    proj = ff.GetProjection()
    ff = None
    
    return geotrans, proj

#Function to save an array as a pcraster map
def save_raster_file(savedir, outfilename, numcols, numrows, dataarray, geotrans, proj):    
    driver = gdal.GetDriverByName('GTiff')
    dst_filename = savedir+outfilename+'.tiff'
    dataset = driver.Create(dst_filename, numcols, numrows, 1, gdal.GDT_Float32)
    dataset.GetRasterBand(1).WriteArray(dataarray)    
    dataset.SetGeoTransform(geotrans)
    dataset.SetProjection(proj)
    dataset.FlushCache()
    dataset=None    
    

#Function to take max probabilities for both 20th and 80th percentiles, and decide which is the one which will be displayed on the interface
# e.g. which comes first / which is most probable criteria
# 20th percentile probabilities are saved with a negative so as to distinguish between probabilities of high flow and low flow (blue vs orange on the map)
def decide_max_prob(val80, val80week, val20, val20week):

    #Firstly, if both the 80th and 20th percentile are exceeded in the forecast horizon with probability >50% (which is the minimum to get a colour),
    #then choose whichever happens first
    if val80 >= 0.5 and val20 >= 0.5:
        if val80week < val20week:
            return val80
        elif val20week < val80week:
            return -val20
        
    #However, if one is above 50% probability and the other below, we go with the one with above 50% probability
    elif val20 >= 0.5 and val80 < 0.5:
        return -val20
    
    elif val80 >= 0.5 and val20 < 0.5:
        return val80
    
    #If neither thresholds have >50% probability of being exceeded, we go with the one with the highest probability?
    elif val80 < 0.5 and val20 < 0.5:
        if val80 > val20:
            return val80
        elif val20 > val80:
            return -val20


#################################
# MAIN FUNCTION
#################################

def compute_max_probs(year, month):

    savedir = "/maxexcprobs/save_dir/"+str(year)+str(month).zfill(2)+"/"
    if not os.path.exists(savedir):
        os.makedirs(savedir)
        
    datadir = "/weekly_exc_probs/savedir/"+str(year)+str(month).zfill(2)+"/"
    
    numcols=3600
    numrows=1500
    numweeks=17
    
    #rrays to hold all the probabilities for all the weeks to use for the calculations 
    allweeksdatapctl20 = np.zeros((numrows,numcols,numweeks))
    allweeksdatapctl80 = np.zeros((numrows,numcols,numweeks))
    
    #arrays to hold (for each grid point) the maximum probability over all 4 months
    Pmax80_all = np.zeros((numrows,numcols))
    Pmax20_all = np.zeros((numrows,numcols))
    
    
    for week in range(0,17):
            
        weeklydatapctl20 = open_raster_file(datadir,year,month,week,20)
        weeklydatapctl80 = open_raster_file(datadir,year,month,week,80)
            
        allweeksdatapctl20[:,:,week] = weeklydatapctl20
        allweeksdatapctl80[:,:,week] = weeklydatapctl80

    #array to hold the overall max probability for this forecast, which will be saved and will be displayed on the interface (assigning colours)
    Pmax_all = np.zeros((numrows,numcols))

    
    print "Starting max prob calculations"
    for x in range(numrows):
        for y in range(numcols):
                
            #Get the maximum probability of exceeding the 80th / 20th percentile across all weeks, and the week in which that happens (in case it exceeds both during the 4 months)
            #Excluding first 2 weeks, because we can't release the product before the 10th of the month
            Pmax80_all[x,y] = max(allweeksdatapctl80[x,y,2:16])
            Pmax80_all_week = np.where(allweeksdatapctl80[x,y,2:16] == max(allweeksdatapctl80[x,y,2:16]))
                
            Pmax20_all[x,y] = max(allweeksdatapctl20[x,y,2:16])
            Pmax20_all_week = np.where(allweeksdatapctl20[x,y,2:16] == max(allweeksdatapctl20[x,y,2:16]))
                    
            #Run the function to decide which of the max probabilities will get displayed (i.e. which is more probable / sooner?)
            Pmax_all[x,y] = decide_max_prob(Pmax80_all[x,y], Pmax80_all_week[0][0], Pmax20_all[x,y], Pmax20_all_week[0][0])

    #save 
    geotrans, proj = get_geo_data(datadir, year, month, 0, 90)    
    save_raster_file(savedir, "max_probs_interface_"+str(year)+str(month).zfill(2)+"_fcst_4month_overview", numcols, numrows, Pmax_all, geotrans, proj)


if __name__ == '__main__':
        compute_max_probs(int(sys.argv[1]),int(sys.argv[2]))

